#
# Generates C++ sources from the .proto files
#
# protobuf_generate_cpp (<SRCS> <HDRS> <DEST> [<ARGN>...])
#
#  SRCS - variable to define with autogenerated source files
#  HDRS - variable to define with autogenerated header files
#  DEST - directory where the source files will be created
#  ARGN - .proto files
#
function(PROTOBUF_GENERATE_CPP SRCS HDRS DEST)
  if(NOT ARGN)
    message(SEND_ERROR "Error: PROTOBUF_GENERATE_CPP() called without any proto files")
    return()
  endif()

  get_filename_component(PROTOBUF_IMPORT_DIRS ${PROTOBUF_IMPORT_DIRS} ABSOLUTE)
  list(APPEND _protobuf_include_path -I ${PROTOBUF_IMPORT_DIRS})

  set(${SRCS})
  set(${HDRS})
  foreach(FIL ${ARGN})
    get_filename_component(FIL ${FIL} ABSOLUTE)
    string(REPLACE "${PROTOBUF_IMPORT_DIRS}/" "" REL_DIR ${FIL})
    get_filename_component(REL_DIR ${REL_DIR} DIRECTORY)
    if(REL_DIR)
      string(APPEND REL_DIR "/")
    endif()
    get_filename_component(FIL_WE ${FIL} NAME_WE)
    set(_dst_src "${DEST}/${REL_DIR}${FIL_WE}.pb.cc")
    set(_dst_hdr "${DEST}/${REL_DIR}${FIL_WE}.pb.h")

    list(APPEND ${SRCS} "${_dst_src}")
    list(APPEND ${HDRS} "${_dst_hdr}")

    add_custom_command(
      OUTPUT "${_dst_src}"
             "${_dst_hdr}"
      COMMAND protobuf::protoc
      ARGS --cpp_out ${DEST} ${PROTOBUF_PROTOC_OPTIONS} ${_protobuf_include_path} ${FIL}
      DEPENDS ${FIL} protobuf::protoc
      COMMENT "Running C++ protocol buffer compiler on ${FIL}"
      VERBATIM )
  endforeach()

  set_source_files_properties(${${SRCS}} ${${HDRS}} PROPERTIES GENERATED TRUE)
  set(${SRCS} ${${SRCS}} PARENT_SCOPE)
  set(${HDRS} ${${HDRS}} PARENT_SCOPE)
endfunction()
