/*
 * Copyright (C) 2017-2020  CZ.NIC, z. s. p. o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <https://www.gnu.org/licenses/>.
 */
/**
 *  @file automatic_keyset_management_i.hh
 *  automatic keyset management corba impl header
 */

//pregenerated by $> omniidl -bcxx -Wba -Wbexample -C./src/bin/corba ~/workspace_18680/enum/idl/idl/RecordStatement.idl

#ifndef SERVER_I_HH_CDE0FA6A11B2420DA0BEF6806B47CDB7
#define SERVER_I_HH_CDE0FA6A11B2420DA0BEF6806B47CDB7

#include "corba/AutomaticKeysetManagement.hh"
#include "src/deprecated/libfred/logger_client.hh"

#include <string>
#include <set>

namespace Fred {
namespace Backend {
namespace AutomaticKeysetManagement {

class AutomaticKeysetManagementImpl; // PIMPL class

} // namespace Fred::Backend::AutomaticKeysetManagement
} // namespace Fred::AutomaticKeysetManagement
} // namespace Fred

namespace Registry {
namespace AutomaticKeysetManagement {

class Server_i: public POA_Registry::AutomaticKeysetManagement::AutomaticKeysetManagementIntf {
public:
    Server_i(
            const std::string& _server_name,
            const std::string& _automatically_managed_keyset_prefix,
            const std::string& _automatically_managed_keyset_registrar,
            const std::string& _automatically_managed_keyset_tech_contact,
            const std::set<std::string>& _automatically_managed_keyset_zones,
            bool _disable_notifier,
            LibFred::Logger::LoggerClient& _logger_client);

    // methods corresponding to defined IDL attributes and operations
    NameserverDomainsSeq* get_nameservers_with_insecure_automatically_managed_domain_candidates();

    NameserverDomainsSeq* get_nameservers_with_secure_automatically_managed_domain_candidates();

    NameserverDomainsSeq* get_nameservers_with_automatically_managed_domains();

    void turn_on_automatic_keyset_management_on_insecure_domain(
            ::CORBA::ULongLong domain_id,
            const Nsset& current_nsset,
            const Keyset& new_keyset);

    void turn_on_automatic_keyset_management_on_secure_domain(
            ::CORBA::ULongLong domain_id,
            const Keyset& new_keyset);

    void update_automatically_managed_keyset_of_domain(
            ::CORBA::ULongLong domain_id,
            const Keyset& new_keyset);

    EmailAddressSeq* get_email_addresses_by_domain_id(
            ::CORBA::ULongLong domain_id);

private:
    // Make sure all instances are built on the heap by making the
    // destructor non-public
    virtual ~Server_i();
    // do not copy
    const std::shared_ptr<Fred::Backend::AutomaticKeysetManagement::AutomaticKeysetManagementImpl> impl_;
    Server_i(const Server_i&);//no body
    Server_i& operator= (const Server_i&);//no body
};

} // namespace Registry::AutomaticKeysetManagement
} // namespace Registry

#endif
